@@ -3,12 +3,11 @@ import knowledge_clustering.kltex as kltex
 import knowledge_clustering.clustering as clust
 import knowledge_clustering.config as config
 import knowledge_clustering.scope_meaning as sm
+import knowledge_clustering.file_updater as fu
 
 
-import hashlib
-import tempfile
 import click
-import os
+from click_default_group import DefaultGroup
 import nltk
 import pkg_resources
 
@@ -16,59 +15,11 @@ ALPHA = 0
 APP_NAME = "knowledge-clustering"
 
 CONFIG_FILE = pkg_resources.resource_filename(
-    "knowledge_clustering", "data/english.txt"
+    "knowledge_clustering", "data/english.ini"
 )
 
 
-def hash_file(filepath):
-    with open(filepath, "rb") as f:
-        file_hash = hashlib.blake2b()
-        chunk = f.read(8192)
-        while chunk:
-            file_hash.update(chunk)
-            chunk = f.read(8192)
-        return file_hash
-
-
-class AtomicUpdate:
-    """
-    A small class using a temporary file to ensure that we have
-    properly replaced the content. Prompts the user if we detect
-    a change in the hash of the file given as input.
-    """
-
-    def __init__(self, filename):
-        self.filename = filename
-        self.hash = hash_file(filename)
-        self.ctx = tempfile.NamedTemporaryFile(mode="w", dir=os.getcwd(), delete=False)
-        self.tmp = None
-
-    def __enter__(self):
-        self.tmp = self.ctx.__enter__()
-        return self.tmp
-
-    def __exit__(self, type, value, traceback):
-        nh = hash_file(self.filename)
-        if nh.hexdigest() != self.hash.hexdigest():
-            print(f"{nh.hexdigest()} ≠ {self.hash.hexdigest()}")
-            b = click.confirm(
-                f"File {self.filename} has been modified\
-                during the run of \
-                the program, erase anyway?",
-                default=None,
-                abort=False,
-                prompt_suffix=": ",
-                show_default=True,
-                err=False,
-            )
-            if b is False:
-                print(f"Temporary file accessible at {self.tmp.name}")
-                return self.ctx.__exit__(type, value, traceback)
-        os.replace(self.tmp.name, self.filename)
-        return self.ctx.__exit__(type, value, traceback)
-
-
-@click.group()
+@click.group(cls=DefaultGroup, default='cluster', default_if_no_args=True)
 def cli():
     """Automated notion clustering for the knowledge LaTeX package"""
     pass
@@ -82,15 +33,21 @@ def init():
 
 
 @cli.command()
-@click.argument(
-    "notion",
+@click.option(
+    "--notion",
+    "-n",
     type=click.Path(
         exists=True, file_okay=True, dir_okay=False, writable=True, readable=True
     ),
+    help="File containing the notions that are already defined.",
+    required=True
 )
-@click.argument(
-    "diagnose",
+@click.option(
+    "--diagnose",
+    "-d",
     type=click.Path(exists=True, file_okay=True, dir_okay=False, readable=True),
+    help="Diagnose file produced by LaTeX.",
+    required=True
 )
 @click.option(
     "--lang",
@@ -104,7 +61,7 @@ def init():
     " /-S",
     default=False,
     help="Print the scopes defined in the notion file and print \
-    the possible meaning of those scope inferred by Knowledge Clustering.",
+the possible meaning of those scope inferred by Knowledge Clustering.",
 )
 @click.option(
     "--config-file",
@@ -116,10 +73,6 @@ def cluster(notion, diagnose, scope, lang, config_file):
     """
     Edit a NOTION file using the knowledges present
     in a DIAGNOSE file.
-
-    NOTION:   File containing the diagnose file produced by TeX.
-
-    DIAGNOSE: File containing the knowledges/notions defined by the user.
     """
     with open(notion, "r") as f:
         document, known_knowledges = kltex.parse(f)
@@ -138,12 +91,13 @@ def cluster(notion, diagnose, scope, lang, config_file):
 
     len_known_knowledges = len(known_knowledges)
     len_bags = [len(bag) for bag in known_knowledges]
+    # update known_knowledges using the clustering algorithm
     clust.clustering(
         known_knowledges,
         unknown_knowledges,
         ALPHA,
         list_prefixes,
-        scopes_meaning,
+        scopes_meaning
     )
     # Compute updated_knowledges and new_knowledges
     new_knowledges = known_knowledges[len_known_knowledges:]
@@ -153,7 +107,7 @@ def cluster(notion, diagnose, scope, lang, config_file):
     ]
     print(f"Found a solution by adding {len(new_knowledges)} new bag(s).")
 
-    with AtomicUpdate(notion) as f:
+    with fu.AtomicUpdate(notion) as f:
         kltex.writeDocument(f, document, updated_knowledges, new_knowledges)
 
 
